@@ -1,23 +1,25 @@
 """
-The most imporant decorator in this module is `@api_view`, which is used
+The most important decorator in this module is `@api_view`, which is used
 for writing function-based views with REST framework.
 
 There are also various decorators for setting the API policies on function
-based views, as well as the `@action` and `@link` decorators, which are
+based views, as well as the `@detail_route` and `@list_route` decorators, which are
 used to annotate methods on viewsets that should be included by routers.
 """
 from __future__ import unicode_literals
-from rest_framework.compat import six
+from django.utils import six
 from rest_framework.views import APIView
 import types
 
 
-def api_view(http_method_names):
+def api_view(http_method_names=None):
 
     """
     Decorator that converts a function-based view into an APIView subclass.
     Takes a list of allowed methods for the view as an argument.
     """
+    if http_method_names is None:
+        http_method_names = ['GET']
 
     def decorator(func):
 
@@ -40,7 +42,7 @@ def api_view(http_method_names):
 
         # api_view applied with eg. string instead of list of strings
         assert isinstance(http_method_names, (list, tuple)), \
-            '@api_view expected a list of strings, recieved %s' % type(http_method_names).__name__
+            '@api_view expected a list of strings, received %s' % type(http_method_names).__name__
 
         allowed_methods = set(http_method_names) | set(('options',))
         WrappedAPIView.http_method_names = [method.lower() for method in allowed_methods]
@@ -107,23 +109,25 @@ def permission_classes(permission_classes):
     return decorator
 
 
-def link(**kwargs):
+def detail_route(methods=['get'], **kwargs):
     """
-    Used to mark a method on a ViewSet that should be routed for GET requests.
+    Used to mark a method on a ViewSet that should be routed for detail requests.
     """
     def decorator(func):
-        func.bind_to_method = 'get'
+        func.bind_to_methods = methods
+        func.detail = True
         func.kwargs = kwargs
         return func
     return decorator
 
 
-def action(**kwargs):
+def list_route(methods=['get'], **kwargs):
     """
-    Used to mark a method on a ViewSet that should be routed for POST requests.
+    Used to mark a method on a ViewSet that should be routed for list requests.
     """
     def decorator(func):
-        func.bind_to_method = 'post'
+        func.bind_to_methods = methods
+        func.detail = False
         func.kwargs = kwargs
         return func
     return decorator
