@@ -7,16 +7,16 @@ The appropriate renderer is called during Django's template response rendering.
 from __future__ import unicode_literals
 from django.core.handlers.wsgi import STATUS_CODE_TEXT
 from django.template.response import SimpleTemplateResponse
-from rest_framework.compat import six
+from django.utils import six
 
 
 class Response(SimpleTemplateResponse):
     """
-    An HttpResponse that allows it's data to be rendered into
+    An HttpResponse that allows its data to be rendered into
     arbitrary media types.
     """
 
-    def __init__(self, data=None, status=200,
+    def __init__(self, data=None, status=None,
                  template_name=None, headers=None,
                  exception=False, content_type=None):
         """
@@ -58,9 +58,15 @@ class Response(SimpleTemplateResponse):
 
         ret = renderer.render(self.data, media_type, context)
         if isinstance(ret, six.text_type):
-            assert charset, 'renderer returned unicode, and did not specify ' \
-            'a charset value.'
+            assert charset, (
+                'renderer returned unicode, and did not specify '
+                'a charset value.'
+            )
             return bytes(ret.encode(charset))
+
+        if not ret:
+            del self['Content-Type']
+
         return ret
 
     @property
